@@ -41,7 +41,7 @@ impl<T: Address> Resolver<T> {
                     format!("{}:{}", req.host(), req.port())
                 };
 
-                let fut = crate::rt::task::spawn_blocking(move || {
+                let fut = crate::rt::spawn_blocking(move || {
                     net::ToSocketAddrs::to_socket_addrs(&host)
                 });
 
@@ -102,22 +102,20 @@ impl<T> Clone for Resolver<T> {
     }
 }
 
-impl<T: Address> ServiceFactory for Resolver<T> {
-    type Request = Connect<T>;
+impl<T: Address, C> ServiceFactory<Connect<T>, C> for Resolver<T> {
     type Response = Connect<T>;
     type Error = ConnectError;
-    type Config = ();
     type Service = Resolver<T>;
     type InitError = ();
     type Future = Ready<Self::Service, Self::InitError>;
 
-    fn new_service(&self, _: ()) -> Self::Future {
+    #[inline]
+    fn new_service(&self, _: C) -> Self::Future {
         Ready::Ok(self.clone())
     }
 }
 
-impl<T: Address> Service for Resolver<T> {
-    type Request = Connect<T>;
+impl<T: Address> Service<Connect<T>> for Resolver<T> {
     type Response = Connect<T>;
     type Error = ConnectError;
     type Future = Pin<Box<dyn Future<Output = Result<Connect<T>, Self::Error>>>>;
