@@ -1,11 +1,8 @@
-use std::{convert::TryFrom, error::Error, fmt, net, rc::Rc, time::Duration};
-
-use bytes::Bytes;
-use futures_core::Stream;
-use serde::Serialize;
+use std::{convert::TryFrom, error::Error, fmt, net, rc::Rc};
 
 #[cfg(feature = "cookie")]
 use coo_kie::{Cookie, CookieJar};
+use serde::Serialize;
 
 use crate::http::body::Body;
 use crate::http::error::HttpError;
@@ -13,6 +10,7 @@ use crate::http::header::{self, HeaderMap, HeaderName, HeaderValue};
 use crate::http::{
     uri, ConnectionType, Method, RequestHead, RequestHeadType, Uri, Version,
 };
+use crate::{time::Millis, util::Bytes, Stream};
 
 use super::error::{FreezeRequestError, InvalidUrl};
 use super::frozen::FrozenClientRequest;
@@ -53,7 +51,7 @@ pub struct ClientRequest {
     #[cfg(feature = "cookie")]
     cookies: Option<CookieJar>,
     response_decompress: bool,
-    timeout: Option<Duration>,
+    timeout: Millis,
     config: Rc<ClientConfig>,
 }
 
@@ -71,7 +69,7 @@ impl ClientRequest {
             addr: None,
             #[cfg(feature = "cookie")]
             cookies: None,
-            timeout: None,
+            timeout: Millis::ZERO,
             response_decompress: true,
         }
         .method(method)
@@ -311,12 +309,12 @@ impl ClientRequest {
         self
     }
 
-    /// Set request timeout. Overrides client wide timeout setting.
+    /// Set request timeout in millis. Overrides client wide timeout setting.
     ///
     /// Request timeout is the total time before a response must be received.
     /// Default value is 5 seconds.
-    pub fn timeout(mut self, timeout: Duration) -> Self {
-        self.timeout = Some(timeout);
+    pub fn timeout<T: Into<Millis>>(mut self, timeout: T) -> Self {
+        self.timeout = timeout.into();
         self
     }
 
@@ -504,10 +502,8 @@ impl ClientRequest {
             if let Some(ref mut jar) = self.cookies {
                 let mut cookie = String::new();
                 for c in jar.delta() {
-                    let name = percent_encode(
-                        c.name().as_bytes(),
-                        crate::http::helpers::USERINFO,
-                    );
+                    let name =
+                        percent_encode(c.name().as_bytes(), crate::http::helpers::USERINFO);
                     let value = percent_encode(
                         c.value().as_bytes(),
                         crate::http::helpers::USERINFO,
@@ -536,8 +532,7 @@ impl ClientRequest {
             } else {
                 #[cfg(any(feature = "compress"))]
                 {
-                    slf =
-                        slf.set_header_if_none(header::ACCEPT_ENCODING, "gzip, deflate")
+                    slf = slf.set_header_if_none(header::ACCEPT_ENCODING, "gzip, deflate")
                 }
             };
         }
