@@ -1,18 +1,17 @@
 use std::sync::atomic::{AtomicBool, AtomicUsize, Ordering};
-use std::task::{Context, Poll};
-use std::{future::Future, pin::Pin, sync::Arc, time};
+use std::{future::Future, pin::Pin, sync::Arc, task::Context, task::Poll};
 
-use tokio::sync::mpsc::{unbounded_channel, UnboundedReceiver, UnboundedSender};
-use tokio::sync::oneshot;
+use async_channel::{unbounded, Receiver, Sender};
+use async_oneshot as oneshot;
+use futures_core::Stream as FutStream;
 
-use crate::rt::time::{sleep_until, Instant, Sleep};
 use crate::rt::{spawn, Arbiter};
-use crate::util::{counter::Counter, join_all};
+use crate::time::{sleep, Millis, Sleep};
+use crate::util::join_all;
 
 use super::accept::{AcceptNotify, Command};
 use super::service::{BoxedServerService, InternalServiceFactory, ServerMessage};
-use super::socket::Stream;
-use super::Token;
+use super::{counter::Counter, socket::Stream, Token};
 
 #[derive(Debug)]
 pub(super) struct WorkerCommand(Connection);
@@ -31,6 +30,7 @@ pub(super) struct Connection {
     pub(super) token: Token,
 }
 
+const STOP_TIMEOUT: Millis = Millis::ONE_SEC;
 static MAX_CONNS: AtomicUsize = AtomicUsize::new(25600);
 
 /// Sets the maximum per-worker number of concurrent connections.
@@ -55,16 +55,16 @@ thread_local! {
 #[derive(Clone, Debug)]
 pub(super) struct WorkerClient {
     pub(super) idx: usize,
-    tx1: UnboundedSender<WorkerCommand>,
-    tx2: UnboundedSender<StopCommand>,
+    tx1: Sender<WorkerCommand>,
+    tx2: Sender<StopCommand>,
     avail: WorkerAvailability,
 }
 
 impl WorkerClient {
     pub(super) fn new(
         idx: usize,
-        tx1: UnboundedSender<WorkerCommand>,
-        tx2: UnboundedSender<StopCommand>,
+        tx1: Sender<WorkerCommand>,
+        tx2: Sender<StopCommand>,
         avail: WorkerAvailability,
     ) -> Self {
         WorkerClient {
@@ -76,7 +76,9 @@ impl WorkerClient {
     }
 
     pub(super) fn send(&self, msg: Connection) -> Result<(), Connection> {
-        self.tx1.send(WorkerCommand(msg)).map_err(|msg| msg.0 .0)
+        self.tx1
+            .try_send(WorkerCommand(msg))
+            .map_err(|msg| msg.into_inner().0)
     }
 
     pub(super) fn available(&self) -> bool {
@@ -84,8 +86,8 @@ impl WorkerClient {
     }
 
     pub(super) fn stop(&self, graceful: bool) -> oneshot::Receiver<bool> {
-        let (result, rx) = oneshot::channel();
-        let _ = self.tx2.send(StopCommand { graceful, result });
+        let (result, rx) = oneshot::oneshot();
+        let _ = self.tx2.try_send(StopCommand { graceful, result });
         rx
     }
 }
@@ -121,14 +123,14 @@ impl WorkerAvailability {
 /// Worker accepts Socket objects via unbounded channel and starts stream
 /// processing.
 pub(super) struct Worker {
-    rx: UnboundedReceiver<WorkerCommand>,
-    rx2: UnboundedReceiver<StopCommand>,
+    rx: Receiver<WorkerCommand>,
+    rx2: Receiver<StopCommand>,
     services: Vec<WorkerService>,
     availability: WorkerAvailability,
     conns: Counter,
     factories: Vec<Box<dyn InternalServiceFactory>>,
     state: WorkerState,
-    shutdown_timeout: time::Duration,
+    shutdown_timeout: Millis,
 }
 
 struct WorkerService {
@@ -159,10 +161,10 @@ impl Worker {
         idx: usize,
         factories: Vec<Box<dyn InternalServiceFactory>>,
         availability: WorkerAvailability,
-        shutdown_timeout: time::Duration,
+        shutdown_timeout: Millis,
     ) -> WorkerClient {
-        let (tx1, rx1) = unbounded_channel();
-        let (tx2, rx2) = unbounded_channel();
+        let (tx1, rx1) = unbounded();
+        let (tx2, rx2) = unbounded();
         let avail = availability.clone();
 
         Arbiter::default().exec_fn(move || {
@@ -185,11 +187,11 @@ impl Worker {
     }
 
     async fn create(
-        rx: UnboundedReceiver<WorkerCommand>,
-        rx2: UnboundedReceiver<StopCommand>,
+        rx: Receiver<WorkerCommand>,
+        rx2: Receiver<StopCommand>,
         factories: Vec<Box<dyn InternalServiceFactory>>,
         availability: WorkerAvailability,
-        shutdown_timeout: time::Duration,
+        shutdown_timeout: Millis,
     ) -> Result<Worker, ()> {
         availability.set(false);
         let mut wrk = MAX_CONNS_COUNTER.with(move |conns| Worker {
@@ -317,11 +319,7 @@ enum WorkerState {
         Token,
         Pin<Box<dyn Future<Output = Result<Vec<(Token, BoxedServerService)>, ()>>>>,
     ),
-    Shutdown(
-        Pin<Box<Sleep>>,
-        Pin<Box<Sleep>>,
-        Option<oneshot::Sender<bool>>,
-    ),
+    Shutdown(Sleep, Sleep, Option<oneshot::Sender<bool>>),
 }
 
 impl Future for Worker {
@@ -329,8 +327,10 @@ impl Future for Worker {
 
     fn poll(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
         // `StopWorker` message handler
-        if let Poll::Ready(Some(StopCommand { graceful, result })) =
-            Pin::new(&mut self.rx2).poll_recv(cx)
+        if let Poll::Ready(Some(StopCommand {
+            graceful,
+            mut result,
+        })) = Pin::new(&mut self.rx2).poll_next(cx)
         {
             self.availability.set(false);
             let num = num_connections();
@@ -344,10 +344,8 @@ impl Future for Worker {
                 if num != 0 {
                     info!("Graceful worker shutdown, {} connections", num);
                     self.state = WorkerState::Shutdown(
-                        Box::pin(sleep_until(
-                            Instant::now() + time::Duration::from_secs(1),
-                        )),
-                        Box::pin(sleep_until(Instant::now() + self.shutdown_timeout)),
+                        sleep(STOP_TIMEOUT),
+                        sleep(self.shutdown_timeout),
                         Some(result),
                     );
                 } else {
@@ -421,7 +419,7 @@ impl Future for Worker {
                 }
 
                 // check graceful timeout
-                match t2.as_mut().poll(cx) {
+                match t2.poll_elapsed(cx) {
                     Poll::Pending => (),
                     Poll::Ready(_) => {
                         let _ = tx.take().unwrap().send(false);
@@ -432,13 +430,11 @@ impl Future for Worker {
                 }
 
                 // sleep for 1 second and then check again
-                match t1.as_mut().poll(cx) {
+                match t1.poll_elapsed(cx) {
                     Poll::Pending => (),
                     Poll::Ready(_) => {
-                        *t1 = Box::pin(sleep_until(
-                            Instant::now() + time::Duration::from_secs(1),
-                        ));
-                        let _ = t1.as_mut().poll(cx);
+                        *t1 = sleep(STOP_TIMEOUT);
+                        let _ = t1.poll_elapsed(cx);
                     }
                 }
                 Poll::Pending
@@ -459,8 +455,7 @@ impl Future for Worker {
                                 self.factories[idx].name(token)
                             );
                             self.availability.set(false);
-                            self.services[token.0].status =
-                                WorkerServiceStatus::Restarting;
+                            self.services[token.0].status = WorkerServiceStatus::Restarting;
                             self.state = WorkerState::Restarting(
                                 idx,
                                 token,
@@ -470,7 +465,7 @@ impl Future for Worker {
                         }
                     }
 
-                    match Pin::new(&mut self.rx).poll_recv(cx) {
+                    match Pin::new(&mut self.rx).poll_next(cx) {
                         // handle incoming io stream
                         Poll::Ready(Some(WorkerCommand(msg))) => {
                             let guard = self.conns.get();
@@ -500,7 +495,7 @@ mod tests {
     use std::sync::{Arc, Mutex};
 
     use super::*;
-    use crate::rt::net::TcpStream;
+    use crate::io::Io;
     use crate::server::service::Factory;
     use crate::service::{Service, ServiceFactory};
     use crate::util::{lazy, Ready};
@@ -518,12 +513,10 @@ mod tests {
         counter: Arc<Mutex<usize>>,
     }
 
-    impl ServiceFactory for SrvFactory {
-        type Request = TcpStream;
+    impl ServiceFactory<Io> for SrvFactory {
         type Response = ();
         type Error = ();
         type Service = Srv;
-        type Config = ();
         type InitError = ();
         type Future = Ready<Srv, ()>;
 
@@ -540,8 +533,7 @@ mod tests {
         st: Arc<Mutex<St>>,
     }
 
-    impl Service for Srv {
-        type Request = TcpStream;
+    impl Service<Io> for Srv {
         type Response = ();
         type Error = ();
         type Future = Ready<(), ()>;
@@ -565,7 +557,7 @@ mod tests {
             }
         }
 
-        fn call(&self, _: TcpStream) -> Self::Future {
+        fn call(&self, _: Io) -> Self::Future {
             Ready::Ok(())
         }
     }
@@ -573,11 +565,11 @@ mod tests {
     #[crate::rt_test]
     #[allow(clippy::mutex_atomic)]
     async fn basics() {
-        let (_tx1, rx1) = unbounded_channel();
-        let (tx2, rx2) = unbounded_channel();
+        let (_tx1, rx1) = unbounded();
+        let (tx2, rx2) = unbounded();
         let (sync_tx, _sync_rx) = std::sync::mpsc::channel();
-        let poll = mio::Poll::new().unwrap();
-        let waker = Arc::new(mio::Waker::new(poll.registry(), mio::Token(1)).unwrap());
+        let poll = Arc::new(polling::Poller::new().unwrap());
+        let waker = poll.clone();
         let avail =
             WorkerAvailability::new(AcceptNotify::new(waker.clone(), sync_tx.clone()));
 
@@ -595,11 +587,11 @@ mod tests {
             vec![Factory::create(
                 "test".to_string(),
                 Token(0),
-                move || f.clone(),
+                move |_| f.clone(),
                 "127.0.0.1:8080".parse().unwrap(),
             )],
             avail.clone(),
-            time::Duration::from_secs(5),
+            Millis(5_000),
         )
         .await
         .unwrap();
@@ -639,8 +631,8 @@ mod tests {
         // shutdown
         let g = MAX_CONNS_COUNTER.with(|conns| conns.get());
 
-        let (tx, rx) = oneshot::channel();
-        tx2.send(StopCommand {
+        let (tx, rx) = oneshot::oneshot();
+        tx2.try_send(StopCommand {
             graceful: true,
             result: tx,
         })
@@ -653,8 +645,8 @@ mod tests {
         let _ = rx.await;
 
         // force shutdown
-        let (_tx1, rx1) = unbounded_channel();
-        let (tx2, rx2) = unbounded_channel();
+        let (_tx1, rx1) = unbounded();
+        let (tx2, rx2) = unbounded();
         let avail = WorkerAvailability::new(AcceptNotify::new(waker, sync_tx.clone()));
         let f = SrvFactory {
             st: st.clone(),
@@ -667,11 +659,11 @@ mod tests {
             vec![Factory::create(
                 "test".to_string(),
                 Token(0),
-                move || f.clone(),
+                move |_| f.clone(),
                 "127.0.0.1:8080".parse().unwrap(),
             )],
             avail.clone(),
-            time::Duration::from_secs(5),
+            Millis(5_000),
         )
         .await
         .unwrap();
@@ -683,8 +675,8 @@ mod tests {
         let _ = lazy(|cx| Pin::new(&mut worker).poll(cx)).await;
         assert!(avail.available());
 
-        let (tx, rx) = oneshot::channel();
-        tx2.send(StopCommand {
+        let (tx, rx) = oneshot::oneshot();
+        tx2.try_send(StopCommand {
             graceful: false,
             result: tx,
         })
