@@ -475,7 +475,7 @@ where
         Error = Err::Container,
         InitError = (),
     > {
-        map_config(self.into_factory(), move |_| Default::default())
+        IntoServiceFactory::<AppFactory<M, F, Err>, Request, ()>::into_factory(self)
     }
 
     /// Construct service factory suitable for `http::HttpService`.
@@ -505,7 +505,18 @@ where
         Error = Err::Container,
         InitError = (),
     > {
-        map_config(self.into_factory(), move |_| cfg.clone())
+        let app = AppFactory {
+            filter: self.filter,
+            middleware: Rc::new(self.middleware),
+            data: Rc::new(self.data),
+            data_factories: Rc::new(self.data_factories),
+            services: Rc::new(RefCell::new(self.services)),
+            external: RefCell::new(self.external),
+            default: self.default,
+            extensions: RefCell::new(Some(self.extensions)),
+            case_insensitive: self.case_insensitive,
+        };
+        map_config(app, move |_| cfg.clone())
     }
 }
 
@@ -538,6 +549,34 @@ where
     }
 }
 
+impl<M, F, Err> IntoServiceFactory<AppFactory<M, F, Err>, Request, ()> for App<M, F, Err>
+where
+    M: Transform<AppService<F::Service, Err>> + 'static,
+    M::Service: Service<WebRequest<Err>, Response = WebResponse, Error = Err::Container>,
+    F: ServiceFactory<
+        WebRequest<Err>,
+        Response = WebRequest<Err>,
+        Error = Err::Container,
+        InitError = (),
+    >,
+    F::Future: 'static,
+    Err: ErrorRenderer,
+{
+    fn into_factory(self) -> AppFactory<M, F, Err> {
+        AppFactory {
+            filter: self.filter,
+            middleware: Rc::new(self.middleware),
+            data: Rc::new(self.data),
+            data_factories: Rc::new(self.data_factories),
+            services: Rc::new(RefCell::new(self.services)),
+            external: RefCell::new(self.external),
+            default: self.default,
+            extensions: RefCell::new(Some(self.extensions)),
+            case_insensitive: self.case_insensitive,
+        }
+    }
+}
+
 pub struct Stack<Inner, Outer> {
     inner: Inner,
     outer: Outer,
