@@ -322,15 +322,13 @@ where
             }
 
             if encoding == UTF_8 {
-                serde_urlencoded::from_bytes::<U>(&body)
-                    .map_err(|_| UrlencodedError::Parse)
+                serde_urlencoded::from_bytes::<U>(&body).map_err(|_| UrlencodedError::Parse)
             } else {
                 let body = encoding
                     .decode_without_bom_handling_and_without_replacement(&body)
                     .map(|s| s.into_owned())
                     .ok_or(UrlencodedError::Parse)?;
-                serde_urlencoded::from_str::<U>(&body)
-                    .map_err(|_| UrlencodedError::Parse)
+                serde_urlencoded::from_str::<U>(&body).map_err(|_| UrlencodedError::Parse)
             }
         }));
         self.poll(cx)
