@@ -13,11 +13,11 @@ use ntex::http::client::error::{JsonPayloadError, SendRequestError};
 use ntex::http::client::{Client, Connector};
 use ntex::http::test::server as test_server;
 use ntex::http::{header, HttpMessage, HttpService};
-use ntex::service::{map_config, pipeline_factory, Service};
+use ntex::service::{map_config, pipeline_factory};
 use ntex::web::dev::AppConfig;
 use ntex::web::middleware::Compress;
 use ntex::web::{self, test, App, BodyEncoding, Error, HttpRequest, HttpResponse};
-use ntex::{time::Millis, time::Seconds, util::Bytes};
+use ntex::{time::sleep, time::Millis, time::Seconds, util::Bytes};
 
 const STR: &str = "Hello World Hello World Hello World Hello World Hello World \
                    Hello World Hello World Hello World Hello World Hello World \
@@ -162,16 +162,13 @@ async fn test_form() {
 async fn test_timeout() {
     let srv = test::server(|| {
         App::new().service(web::resource("/").route(web::to(|| async {
-            ntex::time::sleep(Millis(2000)).await;
+            sleep(Millis(2000)).await;
             HttpResponse::Ok().body(STR)
         })))
     });
 
     let connector = Connector::default()
-        .connector(
-            ntex::connect::Connector::new()
-                .map(|sock| (sock, ntex::http::Protocol::Http1)),
-        )
+        .connector(ntex::connect::Connector::new())
         .timeout(Seconds(15))
         .finish();
 
@@ -191,7 +188,7 @@ async fn test_timeout() {
 async fn test_timeout_override() {
     let srv = test::server(|| {
         App::new().service(web::resource("/").route(web::to(|| async {
-            ntex::time::sleep(Millis(2000)).await;
+            sleep(Millis(2000)).await;
             HttpResponse::Ok().body(STR)
         })))
     });
@@ -215,15 +212,12 @@ async fn test_connection_reuse() {
             num2.fetch_add(1, Ordering::Relaxed);
             ok(io)
         })
-        .and_then(
-            HttpService::new(map_config(
-                App::new().service(
-                    web::resource("/").route(web::to(|| async { HttpResponse::Ok() })),
-                ),
-                |_| AppConfig::default(),
-            ))
-            .tcp(),
-        )
+        .and_then(HttpService::new(map_config(
+            App::new().service(
+                web::resource("/").route(web::to(|| async { HttpResponse::Ok() })),
+            ),
+            |_| AppConfig::default(),
+        )))
     });
 
     let client = Client::build().timeout(Seconds(10)).finish();
@@ -253,15 +247,12 @@ async fn test_connection_force_close() {
             num2.fetch_add(1, Ordering::Relaxed);
             ok(io)
         })
-        .and_then(
-            HttpService::new(map_config(
-                App::new().service(
-                    web::resource("/").route(web::to(|| async { HttpResponse::Ok() })),
-                ),
-                |_| AppConfig::default(),
-            ))
-            .tcp(),
-        )
+        .and_then(HttpService::new(map_config(
+            App::new().service(
+                web::resource("/").route(web::to(|| async { HttpResponse::Ok() })),
+            ),
+            |_| AppConfig::default(),
+        )))
     });
 
     let client = Client::build().timeout(Seconds(10)).finish();
@@ -272,6 +263,7 @@ async fn test_connection_force_close() {
     assert!(response.status().is_success());
 
     // req 2
+    let client = Client::build().timeout(Seconds(10)).finish();
     let req = client.post(srv.url("/")).force_close();
     let response = req.send().await.unwrap();
     assert!(response.status().is_success());
@@ -291,15 +283,12 @@ async fn test_connection_server_close() {
             num2.fetch_add(1, Ordering::Relaxed);
             ok(io)
         })
-        .and_then(
-            HttpService::new(map_config(
-                App::new().service(web::resource("/").route(web::to(|| async {
-                    HttpResponse::Ok().force_close().finish()
-                }))),
-                |_| AppConfig::default(),
-            ))
-            .tcp(),
-        )
+        .and_then(HttpService::new(map_config(
+            App::new().service(web::resource("/").route(web::to(|| async {
+                HttpResponse::Ok().force_close().finish()
+            }))),
+            |_| AppConfig::default(),
+        )))
     });
 
     let client = Client::build().timeout(Seconds(10)).finish();
@@ -329,16 +318,13 @@ async fn test_connection_wait_queue() {
             num2.fetch_add(1, Ordering::Relaxed);
             ok(io)
         })
-        .and_then(
-            HttpService::new(map_config(
-                App::new().service(
-                    web::resource("/")
-                        .route(web::to(|| async { HttpResponse::Ok().body(STR) })),
-                ),
-                |_| AppConfig::default(),
-            ))
-            .tcp(),
-        )
+        .and_then(HttpService::new(map_config(
+            App::new().service(
+                web::resource("/")
+                    .route(web::to(|| async { HttpResponse::Ok().body(STR) })),
+            ),
+            |_| AppConfig::default(),
+        )))
     });
 
     let client = Client::build()
@@ -378,15 +364,12 @@ async fn test_connection_wait_queue_force_close() {
             num2.fetch_add(1, Ordering::Relaxed);
             ok(io)
         })
-        .and_then(
-            HttpService::new(map_config(
-                App::new().service(web::resource("/").route(web::to(|| async {
-                    HttpResponse::Ok().force_close().body(STR)
-                }))),
-                |_| AppConfig::default(),
-            ))
-            .tcp(),
-        )
+        .and_then(HttpService::new(map_config(
+            App::new().service(web::resource("/").route(web::to(|| async {
+                HttpResponse::Ok().force_close().body(STR)
+            }))),
+            |_| AppConfig::default(),
+        )))
     });
 
     let client = Client::build()
@@ -824,11 +807,11 @@ async fn client_read_until_eof() {
             }
         }
     });
-    ntex::time::sleep(Millis(300)).await;
+    sleep(Millis(300)).await;
 
     // client request
     let req = Client::build()
-        .timeout(Seconds(30))
+        .timeout(Seconds(5))
         .finish()
         .get(format!("http://{}/", addr).as_str());
     let mut response = req.send().await.unwrap();
