@@ -1,11 +1,10 @@
-use std::sync::mpsc;
-use std::{thread, time::Duration};
+use std::{sync::mpsc, thread, time::Duration};
 
 #[cfg(feature = "openssl")]
-use open_ssl::ssl::SslAcceptorBuilder;
+use tls_openssl::ssl::SslAcceptorBuilder;
 
 use ntex::web::{self, App, HttpResponse, HttpServer};
-use ntex::{server::TestServer, time::Seconds};
+use ntex::{rt, server::TestServer, time::sleep, time::Seconds};
 
 #[cfg(unix)]
 #[ntex::test]
@@ -14,10 +13,10 @@ async fn test_run() {
     let (tx, rx) = mpsc::channel();
 
     thread::spawn(move || {
-        let mut sys = ntex::rt::System::new("test");
+        let sys = ntex::rt::System::new("test");
 
-        let srv = sys.exec(|| {
-            HttpServer::new(|| {
+        sys.run(move || {
+            let srv = HttpServer::new(|| {
                 App::new().service(
                     web::resource("/")
                         .route(web::to(|| async { HttpResponse::Ok().body("test") })),
@@ -36,11 +35,10 @@ async fn test_run() {
             .disable_signals()
             .bind(format!("{}", addr))
             .unwrap()
-            .run()
-        });
-
-        let _ = tx.send((srv, ntex::rt::System::current()));
-        let _ = sys.run();
+            .run();
+            let _ = tx.send((srv, ntex::rt::System::current()));
+            Ok(())
+        })
     });
     let (srv, sys) = rx.recv().unwrap();
 
@@ -63,7 +61,7 @@ async fn test_run() {
 
 #[cfg(feature = "openssl")]
 fn ssl_acceptor() -> std::io::Result<SslAcceptorBuilder> {
-    use open_ssl::ssl::{SslAcceptor, SslFiletype, SslMethod, SslVerifyMode};
+    use tls_openssl::ssl::{SslAcceptor, SslFiletype, SslMethod, SslVerifyMode};
     // load ssl keys
     let mut builder = SslAcceptor::mozilla_intermediate(SslMethod::tls()).unwrap();
     builder.set_verify(SslVerifyMode::NONE);
@@ -78,7 +76,7 @@ fn ssl_acceptor() -> std::io::Result<SslAcceptorBuilder> {
 
 #[cfg(feature = "openssl")]
 fn client() -> ntex::http::client::Client {
-    use open_ssl::ssl::{SslConnector, SslMethod, SslVerifyMode};
+    use tls_openssl::ssl::{SslConnector, SslMethod, SslVerifyMode};
     let mut builder = SslConnector::builder(SslMethod::tls()).unwrap();
     builder.set_verify(SslVerifyMode::NONE);
     let _ = builder
@@ -105,11 +103,11 @@ async fn test_openssl() {
     let (tx, rx) = mpsc::channel();
 
     thread::spawn(move || {
-        let mut sys = ntex::rt::System::new("test");
+        let sys = ntex::rt::System::new("test");
         let builder = ssl_acceptor().unwrap();
 
-        let srv = sys.exec(|| {
-            HttpServer::new(|| {
+        sys.run(move || {
+            let srv = HttpServer::new(|| {
                 App::new().service(web::resource("/").route(web::to(
                     |req: HttpRequest| async move {
                         assert!(req.app_config().secure());
@@ -123,11 +121,10 @@ async fn test_openssl() {
             .disable_signals()
             .bind_openssl(format!("{}", addr), builder)
             .unwrap()
-            .run()
-        });
-
-        let _ = tx.send((srv, ntex::rt::System::current()));
-        let _ = sys.run();
+            .run();
+            let _ = tx.send((srv, ntex::rt::System::current()));
+            Ok(())
+        })
     });
     let (srv, sys) = rx.recv().unwrap();
 
@@ -146,18 +143,17 @@ async fn test_openssl() {
 #[ntex::test]
 #[cfg(all(feature = "rustls", feature = "openssl"))]
 async fn test_rustls() {
-    use std::fs::File;
-    use std::io::BufReader;
+    use std::{fs::File, io::BufReader};
 
     use ntex::web::HttpRequest;
-    use rust_tls::{Certificate, PrivateKey, ServerConfig as RustlsServerConfig};
-    use rustls_pemfile::{certs, pkcs8_private_keys};
+    use rustls_pemfile::{certs, rsa_private_keys};
+    use tls_rustls::{Certificate, PrivateKey, ServerConfig as RustlsServerConfig};
 
     let addr = TestServer::unused_addr();
     let (tx, rx) = mpsc::channel();
 
     thread::spawn(move || {
-        let mut sys = ntex::rt::System::new("test");
+        let sys = ntex::rt::System::new("test");
 
         // load ssl keys
         let cert_file = &mut BufReader::new(File::open("./tests/cert.pem").unwrap());
@@ -167,15 +163,15 @@ async fn test_rustls() {
             .iter()
             .map(|c| Certificate(c.to_vec()))
             .collect();
-        let keys = PrivateKey(pkcs8_private_keys(key_file).unwrap().remove(0));
+        let keys = PrivateKey(rsa_private_keys(key_file).unwrap().remove(0));
         let config = RustlsServerConfig::builder()
             .with_safe_defaults()
             .with_no_client_auth()
             .with_single_cert(cert_chain, keys)
             .unwrap();
 
-        let srv = sys.exec(|| {
-            HttpServer::new(|| {
+        sys.run(move || {
+            let srv = HttpServer::new(|| {
                 App::new().service(web::resource("/").route(web::to(
                     |req: HttpRequest| async move {
                         assert!(req.app_config().secure());
@@ -189,11 +185,10 @@ async fn test_rustls() {
             .disable_signals()
             .bind_rustls(format!("{}", addr), config)
             .unwrap()
-            .run()
-        });
-
-        let _ = tx.send((srv, ntex::rt::System::current()));
-        let _ = sys.run();
+            .run();
+            let _ = tx.send((srv, ntex::rt::System::current()));
+            Ok(())
+        })
     });
     let (srv, sys) = rx.recv().unwrap();
 
@@ -205,7 +200,7 @@ async fn test_rustls() {
     // stop
     let _ = srv.stop(false);
 
-    thread::sleep(Duration::from_millis(100));
+    sleep(Duration::from_millis(100)).await;
     sys.stop();
 }
 
@@ -215,10 +210,10 @@ async fn test_bind_uds() {
     let (tx, rx) = mpsc::channel();
 
     thread::spawn(move || {
-        let mut sys = ntex::rt::System::new("test");
+        let sys = ntex::rt::System::new("test");
 
-        let srv = sys.exec(|| {
-            HttpServer::new(|| {
+        sys.run(move || {
+            let srv = HttpServer::new(|| {
                 App::new().service(
                     web::resource("/")
                         .route(web::to(|| async { HttpResponse::Ok().body("test") })),
@@ -230,11 +225,10 @@ async fn test_bind_uds() {
             .disable_signals()
             .bind_uds("/tmp/uds-test")
             .unwrap()
-            .run()
-        });
-
-        let _ = tx.send((srv, ntex::rt::System::current()));
-        let _ = sys.run();
+            .run();
+            let _ = tx.send((srv, ntex::rt::System::current()));
+            Ok(())
+        })
     });
     let (srv, sys) = rx.recv().unwrap();
 
@@ -244,9 +238,7 @@ async fn test_bind_uds() {
         .connector(
             client::Connector::default()
                 .connector(ntex::service::fn_service(|_| async {
-                    let stream =
-                        ntex::rt::net::UnixStream::connect("/tmp/uds-test").await?;
-                    Ok((stream, ntex::http::Protocol::Http1))
+                    Ok(rt::unix_connect("/tmp/uds-test").await?)
                 }))
                 .finish(),
         )
@@ -257,7 +249,7 @@ async fn test_bind_uds() {
     // stop
     let _ = srv.stop(false);
 
-    thread::sleep(Duration::from_millis(100));
+    sleep(Duration::from_millis(100)).await;
     sys.stop();
 }
 
@@ -267,12 +259,13 @@ async fn test_listen_uds() {
     let (tx, rx) = mpsc::channel();
 
     thread::spawn(move || {
-        let mut sys = ntex::rt::System::new("test");
+        let sys = ntex::rt::System::new("test");
 
-        let srv = sys.exec(|| {
+        sys.run(move || {
+            let _ = std::fs::remove_file("/tmp/uds-test2");
             let lst = std::os::unix::net::UnixListener::bind("/tmp/uds-test2").unwrap();
 
-            HttpServer::new(|| {
+            let srv = HttpServer::new(|| {
                 App::new().service(
                     web::resource("/")
                         .route(web::to(|| async { HttpResponse::Ok().body("test") })),
@@ -284,11 +277,10 @@ async fn test_listen_uds() {
             .disable_signals()
             .listen_uds(lst)
             .unwrap()
-            .run()
-        });
-
-        let _ = tx.send((srv, ntex::rt::System::current()));
-        let _ = sys.run();
+            .run();
+            let _ = tx.send((srv, ntex::rt::System::current()));
+            Ok(())
+        })
     });
     let (srv, sys) = rx.recv().unwrap();
 
@@ -298,9 +290,7 @@ async fn test_listen_uds() {
         .connector(
             client::Connector::default()
                 .connector(ntex::service::fn_service(|_| async {
-                    let stream =
-                        ntex::rt::net::UnixStream::connect("/tmp/uds-test2").await?;
-                    Ok((stream, ntex::http::Protocol::Http1))
+                    Ok(rt::unix_connect("/tmp/uds-test2").await?)
                 }))
                 .finish(),
         )
@@ -311,6 +301,6 @@ async fn test_listen_uds() {
     // stop
     let _ = srv.stop(false);
 
-    thread::sleep(Duration::from_millis(100));
+    sleep(Duration::from_millis(100)).await;
     sys.stop();
 }
